from ..api import Api

from prefect import Task
from prefect.utilities.tasks import defaults_from_attrs

from typing import Any


class Update(Task):
    def __init__(self, aws_secret_name: str = None, aws_secret_region: str = None, event: dict = None, **kwargs: Any):
        self.aws_secret_region = aws_secret_region
        self.aws_secret_name = aws_secret_name
        self.event = event
        super().__init__(**kwargs)

    @defaults_from_attrs("event", "aws_secret_region", "aws_secret_name")
    def run(self, aws_secret_name: str = None, aws_secret_region: str = None, event: dict = None) -> dict:

        if not event:
            raise ValueError('An event must be provided.')

        try:
            wc_api = Api(aws_secret_name=aws_secret_name, aws_secret_region=aws_secret_region)
            resource = wc_api.api_client.put(endpoint=event.get('endpoint', None),
                                             data=event.get('data', None) or dict(),
                                             **event.get('kwargs', None) or dict())

        except Exception as e:
            print(e)
            raise e

        return resource.json()



import os
from configparser import NoOptionError, NoSectionError, ConfigParser

from exactonline.storage import ExactOnlineConfig, MissingSetting

from prefect.configuration import load_toml


class S3Storage(ExactOnlineConfig, ConfigParser):
    """
    Configuration based on the SafeConfigParser and the
    ExactOnlineConfig.

    Takes an S3 key as input and writes/reads that file
    """
    def __init__(self, key, config_path, s3_client, **kwargs):
        super(S3Storage, self).__init__(**kwargs)

        self.key = key
        self.config_path = config_path
        self.s3_client = s3_client

        os.makedirs(os.path.dirname(self.config_path), exist_ok=True)

        if hasattr(self.config_path, 'read'):
            self.overwrite = False
        else:
            self.overwrite = self.config_path

    def read_config(self):
        if hasattr(self.config_path, 'read'):
            if hasattr(self, 'read_file'):
                self.read_file(self.config_path)
            else:
                self.readfp(self.config_path)
        else:
            self.read([self.config_path])

    def get(self, section, option, **kwargs):
        """
        Get method that raises MissingSetting if the value was unset.

        This differs from the SafeConfigParser which may raise either a
        NoOptionError or a NoSectionError.

        We take extra **kwargs because the Python 3.5 configparser extends the
        get method signature and it calls self with those parameters.

            def get(self, section, option, *, raw=False, vars=None,
                    fallback=_UNSET):
        """
        try:
            ret = super(ExactOnlineConfig, self).get(section, option, **kwargs)
        except (NoOptionError, NoSectionError):
            raise MissingSetting(option, section)

        return ret

    def set(self, section, option, value: str = None):
        """
        Set method that (1) auto-saves if possible and (2) auto-creates
        sections.
        """
        try:
            super(ExactOnlineConfig, self).set(section, option, value)
        except NoSectionError:
            self.add_section(section)
            super(ExactOnlineConfig, self).set(section, option, value)

        # Save automatically!
        self.save()

    def save(self):
        if self.overwrite:
            with open(self.overwrite, 'w') as output:
                self.write(output)
            self.s3_client.upload_file(self.overwrite, self.key)



import os
import json
from configparser import NoOptionError, NoSectionError, ConfigParser
from time import time


class MissingSetting(ValueError):
    pass


class SecretsManagerStorage(ConfigParser):
    """
    Configuration based on the SafeConfigParser and the
    ExactOnlineConfig.

    Takes a Secret Manager Secret name as input and writes/reads that Secret
    """
    def __init__(self, secretsmanager_client, name, secret=None, **kwargs):
        super(SecretsManagerStorage, self).__init__(**kwargs)

        self.secretsmanager_client = secretsmanager_client
        self.name = name
        self.secret = secret

    def read_config(self):
        secret_response = self.secretsmanager_client.get_value()
        self.secret = json.loads(secret_response.get('SecretString'))
        self.read_dict(self.secret)

    def get(self, section, option, **kwargs):
        """
        Get method that raises MissingSetting if the value was unset.

        This differs from the SafeConfigParser which may raise either a
        NoOptionError or a NoSectionError.

        We take extra **kwargs because the Python 3.5 configparser extends the
        get method signature and it calls self with those parameters.

            def get(self, section, option, *, raw=False, vars=None,
                    fallback=_UNSET):
        """
        try:
            ret = super(ConfigParser, self).get(section, option, **kwargs)
        except (NoOptionError, NoSectionError):
            raise MissingSetting(option, section)

        return ret

    def set(self, section, option, value: str = None):
        """
        Set method that (1) auto-saves if possible and (2) auto-creates
        sections.
        """
        try:
            super(ConfigParser, self).set(section, option, value)
        except NoSectionError:
            self.add_section(section)
            super(ConfigParser, self).set(section, option, value)

        # Save automatically!
        # self.save()




import json
import os
from time import time

from configparser import ConfigParser
from .aws import SecretsManagerStorage, SecretsManagerClient

from woocommerce import API

from prefect import *

from typing import Any


class Api:
    _instance = None

    def __new__(cls, aws_secret_name: str = None, aws_secret_region: str = None):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, aws_secret_name: str = None, aws_secret_region: str = None):
        try:
            if os.getenv('AWS_CONFIG_FILE'):
                self.aws_config = read_config(os.getenv('AWS_CONFIG_FILE'))

            self.aws_access_key = os.getenv('AWS_ACCESS_KEY_ID')
            self.aws_secret_key = os.getenv('AWS_SECRET_ACCESS_KEY')

            if aws_secret_name:
                self.aws_secretsmanager_secret_name = aws_secret_name
            else:
                self.aws_secretsmanager_secret_name = os.getenv('AWS_SM_WOOCOMMERCE_SECRET_NAME')

            if aws_secret_region:
                self.aws_secretsmanager_region = aws_secret_region
            else:
                self.aws_secretsmanager_region = os.getenv('AWS_SM_REGION')

            self.secretsmanager_client = SecretsManagerClient.get_instance(
                self.aws_access_key, self.aws_secret_key,
                region_name=self.aws_secretsmanager_region,
            )

            self.secretsmanager_client.name = self.aws_secretsmanager_secret_name

            self.storage = SecretsManagerStorage(secretsmanager_client=self.secretsmanager_client,
                                                 name=self.secretsmanager_client.name)

            self.storage.read_config()

            self.url = self.storage.get('woocommerce', 'url')
            self.consumer_key = self.storage.get('woocommerce', 'consumer_key')
            self.consumer_secret = self.storage.get('woocommerce', 'consumer_secret')

            kw = dict(self.storage.items('woocommerce'))
            kw.pop('url', None)
            kw.pop('consumer_key', None)
            kw.pop('consumer_secret', None)

            self.api_client = API(url=self.url, consumer_key=self.consumer_key, consumer_secret=self.consumer_secret, **kw)

        except Exception as e:
            print(e)
            raise e


def read_config(filepath):
    """Read AWS configuration file"""
    config = ConfigParser()
    config.read(filepath)
    return {s: dict(config.items(s)) for s in config.sections()}



import boto3


class S3Client:
    """Client for AWS S3"""

    _instance = None

    def __init__(self, s3_client, bucket):
        self.s3_client = s3_client
        self.bucket = bucket

    @staticmethod
    def get_instance(access_key, secret_key, region_name='us-west-2', bucket_name='bucket'):
        if S3Client._instance is None:
            s3_client = boto3.resource(
                's3',
                region_name=region_name,
                aws_access_key_id=access_key,
                aws_secret_access_key=secret_key
            )
            bucket = s3_client.Bucket(bucket_name)
            S3Client._instance = S3Client(s3_client, bucket)
        return S3Client._instance

    def download_file(self, key, dest):
        self.bucket.download_file(key, dest)

    def upload_file(self, src, key):
        self.bucket.Object(key).put(Body=open(src, 'rb'))




import argparse
import base64
import json
import logging
from pprint import pprint
import time
import boto3
from botocore.exceptions import ClientError

logger = logging.getLogger(__name__)


class SecretsManagerClient:
    """Encapsulates Secrets Manager functions."""

    _instance = None

    def __init__(self, secretsmanager_client):
        """
        :param secretsmanager_client: A Boto3 Secrets Manager client.
        """
        self.secretsmanager_client = secretsmanager_client
        self.name = None

    @staticmethod
    def get_instance(access_key, secret_key, region_name='us-west-2'):
        if SecretsManagerClient._instance is None:
            secretsmanager_client = boto3.client(
                'secretsmanager',
                region_name=region_name,
                aws_access_key_id=access_key,
                aws_secret_access_key=secret_key
            )
            SecretsManagerClient._instance = SecretsManagerClient(secretsmanager_client)
        return SecretsManagerClient._instance

    def _clear(self):
        self.name = None

    def create(self, name, secret_value):
        """
        Creates a new secret. The secret value can be a string or bytes.
        :param name: The name of the secret to create.
        :param secret_value: The value of the secret.
        :return: Metadata about the newly created secret.
        """
        self._clear()
        try:
            kwargs = {'Name': name}
            if isinstance(secret_value, str):
                kwargs['SecretString'] = secret_value
            elif isinstance(secret_value, bytes):
                kwargs['SecretBinary'] = secret_value
            response = self.secretsmanager_client.create_secret(**kwargs)
            self.name = name
            logger.info("Created secret %s.", name)
        except ClientError:
            logger.exception("Couldn't get secret %s.", name)
            raise
        else:
            return response

    def describe(self, name=None):
        """
        Gets metadata about a secret.
        :param name: The name of the secret to load. If `name` is None, metadata about
                     the current secret is retrieved.
        :return: Metadata about the secret.
        """
        if self.name is None and name is None:
            raise ValueError
        if name is None:
            name = self.name
        self._clear()
        try:
            response = self.secretsmanager_client.describe_secret(SecretId=name)
            self.name = name
            logger.info("Got secret metadata for %s.", name)
        except ClientError:
            logger.exception("Couldn't get secret metadata for %s.", name)
            raise
        else:
            return response

    def get_value(self, stage=None):
        """
        Gets the value of a secret.
        :param stage: The stage of the secret to retrieve. If this is None, the
                      current stage is retrieved.
        :return: The value of the secret. When the secret is a string, the value is
                 contained in the `SecretString` field. When the secret is bytes,
                 it is contained in the `SecretBinary` field.
        """
        if self.name is None:
            raise ValueError

        try:
            kwargs = {'SecretId': self.name}
            if stage is not None:
                kwargs['VersionStage'] = stage
            response = self.secretsmanager_client.get_secret_value(**kwargs)
            logger.info("Got value for secret %s.", self.name)
        except ClientError:
            logger.exception("Couldn't get value for secret %s.", self.name)
            raise
        else:
            return response

    def get_random_password(self, pw_length):
        """
        Gets a randomly generated password.
        :param pw_length: The length of the password.
        :return: The generated password.
        """
        try:
            response = self.secretsmanager_client.get_random_password(
                PasswordLength=pw_length)
            password = response['RandomPassword']
            logger.info("Got random password.")
        except ClientError:
            logger.exception("Couldn't get random password.")
            raise
        else:
            return password

    def put_value(self, secret_value, stages=None):
        """
        Puts a value into an existing secret. When no stages are specified, the
        value is set as the current ('AWSCURRENT') stage and the previous value is
        moved to the 'AWSPREVIOUS' stage. When a stage is specified that already
        exists, the stage is associated with the new value and removed from the old
        value.
        :param secret_value: The value to add to the secret.
        :param stages: The stages to associate with the secret.
        :return: Metadata about the secret.
        """
        if self.name is None:
            raise ValueError

        try:
            kwargs = {'SecretId': self.name}
            if isinstance(secret_value, str):
                kwargs['SecretString'] = secret_value
            elif isinstance(secret_value, bytes):
                kwargs['SecretBinary'] = secret_value
            if stages is not None:
                kwargs['VersionStages'] = stages
            response = self.secretsmanager_client.put_secret_value(**kwargs)
            logger.info("Value put in secret %s.", self.name)
        except ClientError:
            logger.exception("Couldn't put value in secret %s.", self.name)
            raise
        else:
            return response

    def update_version_stage(self, stage, remove_from, move_to):
        """
        Updates the stage associated with a version of the secret.
        :param stage: The stage to update.
        :param remove_from: The ID of the version to remove the stage from.
        :param move_to: The ID of the version to add the stage to.
        :return: Metadata about the secret.
        """
        if self.name is None:
            raise ValueError

        try:
            response = self.secretsmanager_client.update_secret_version_stage(
                SecretId=self.name, VersionStage=stage, RemoveFromVersionId=remove_from,
                MoveToVersionId=move_to)
            logger.info("Updated version stage %s for secret %s.", stage, self.name)
        except ClientError:
            logger.exception(
                "Couldn't update version stage %s for secret %s.", stage, self.name)
            raise
        else:
            return response

    def delete(self, without_recovery):
        """
        Deletes the secret.
        :param without_recovery: Permanently deletes the secret immediately when True;
                                 otherwise, the deleted secret can be restored within
                                 the recovery window. The default recovery window is
                                 30 days.
        """
        if self.name is None:
            raise ValueError

        try:
            self.secretsmanager_client.delete_secret(
                SecretId=self.name, ForceDeleteWithoutRecovery=without_recovery)
            logger.info("Deleted secret %s.", self.name)
            self._clear()
        except ClientError:
            logger.exception("Deleted secret %s.", self.name)
            raise

    def list(self, max_results):
        """
        Lists secrets for the current account.
        :param max_results: The maximum number of results to return.
        :return: Yields secrets one at a time.
        """
        try:
            paginator = self.secretsmanager_client.get_paginator('list_secrets')
            for page in paginator.paginate(
                    PaginationConfig={'MaxItems': max_results}):
                for secret in page['SecretList']:
                    yield secret
        except ClientError:
            logger.exception("Couldn't list secrets.")
            raise



"""
Tasks for interacting with WooCommerce.
"""
try:
    from woocommerce.api import *
    from .aws import SecretsManagerClient, SecretsManagerStorage
    from .tasks import Create, Get, Update, Delete, Options
except ImportError:
    raise ImportError(
        'Using `prefect.tasks.woocommerce_prefect_tasks` requires Prefect to be installed with the "woocommerce" extra.'
    )



from setuptools import find_packages, setup

import versioneer

with open("README.rst", "r") as fh:
    long_description = fh.read()
with open("requirements.txt", "r") as fh:
    requirements = [line.strip() for line in fh]

setup(
    name='woocommerce_prefect_tasks',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    url='https://github.com/aporacloudmobile/woocommerce_prefect_tasks',
    license='',
    author='Carlos Paiva',
    author_email='carlospaiva87@gmail.com',
    description="",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=requirements,

)
